local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Ui = BaikCore.Ui

-- Add Module
local Visibility = BaikCore:NewModule("Visibility", "AceHook-3.0")
BaikCore.Visibility = Visibility
Visibility.DEFAULT = 3

-- Constants
local STATE_OPTS = {
    "Hide",
    "Hover",
    "Show"
}

local HIDE_IDX = 1
local HOVER_IDX = 2
local SHOW_IDX = 3

-- Private Variables
local visFunc = {}

-- Private Methods
local function HookHover(frame, parent)
    Hook:EnsureHookScript(Visibility, frame, "OnEnter", function(...) parent:SetAlpha(1) end)
    Hook:EnsureHookScript(Visibility, frame, "OnLeave", function(...) parent:SetAlpha(0) end)
end

local function UnhookHover(frame)
    Hook:EnsureUnhook(Visibility, frame, "OnEnter")
    Hook:EnsureUnhook(Visibility, frame, "OnLeave")
end

local function SetVisibilityLocked(dbValue, frame, parent)
    local visibility = Database:Get(dbValue)

    if not parent then
        parent = frame
    end

    if visibility == HIDE_IDX then
        Ui:HideFinal(parent)
        UnhookHover(frame)
    elseif visibility == HOVER_IDX then
        Ui:UndoHideFinal(parent)
        HookHover(frame, parent)
        parent:SetAlpha(0)
    else
        parent:SetAlpha(1)
        Ui:UndoHideFinal(frame)
        UnhookHover(frame)
    end
end

local function SetupLocked()
    for dbValue, func in pairs(visFunc) do
        func()
    end
end

local function Setup()
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

-- Visibility API
function Visibility:SetupFrame(dbValue, frame, parent)
    Assert:String(dbValue)
    Assert:Table(frame)

    if CombatSafe:InLockdown() then
        return
    end

    SetVisibilityLocked(dbValue, frame, parent)
end

function Visibility:SetupFrameAll(dbValue, frameTbl, parent)
    Assert:String(dbValue)
    Assert:Table(frameTbl)

    if CombatSafe:InLockdown() then
        return
    end

    for _, frame in pairs(frameTbl) do
        SetVisibilityLocked(dbValue, frame, parent)
    end
end

function Visibility:GetOpts(dbValue, func)
    visFunc[dbValue] = func

    return {
        name = "Visibility",
        type = "select",
        desc = "Adjusts frame visibility",
        values = STATE_OPTS,
        get = function(...)
            return Database:Get(dbValue)
        end,
        set = function(self, key, value)
            local cur = Database:Get(dbValue)

            -- Update if state changed
            if cur ~= key then
                Database:Set(dbValue, key)

                if not CombatSafe:InLockdown() then
                    func()
                end
            end
        end,
    }
end

-- Ace3 Callbacks
function Visibility:OnInitialize()
    CombatSafe:Setup(SetupLocked)
end

function Visibility:OnEnable()
    Setup()
end

function Visibility:OnDisable()
end
